#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "graph.h"

struct graph *graph_init(int prealloc) {
	struct graph *g = calloc(1, sizeof(*g));
	assert(!(prealloc % graph_alloc_step));
	if (prealloc) {
		g->a_vertices = prealloc;
		g->vertices = calloc(prealloc, sizeof(*g->vertices));
		g->edges = calloc(prealloc * prealloc, sizeof(*g->edges));
	}
	return g;
}


vertex_t graph_get_vertex(struct graph *g, int v) {
	assert(v < g->n_vertices);
	return g->vertices[v];
}

void graph_set_vertex(struct graph *g, int v, vertex_t val) {
	assert(v < g->n_vertices);
	g->vertices[v] = val;
}

int graph_add_vertex(struct graph *g, vertex_t value) {
	int v = g->n_vertices++;
	if (unlikely(v >= g->a_vertices)) {
		int a_v = g->a_vertices + graph_alloc_step;

		g->vertices = realloc(g->vertices, a_v * sizeof(*g->vertices));
		memset(g->vertices + g->a_vertices, (a_v - g->a_vertices) * sizeof(*g->vertices), 0);

		edge_t *edges2 = calloc(a_v * a_v, sizeof(*g->edges));
		for (int i = 0; i < v; i++)
			memcpy(&edges2[i * a_v],
			       &g->edges[i * g->a_vertices],
			       g->a_vertices);
		free(g->edges);
		g->edges = edges2;
	}
	g->vertices[v] = value;
	return v;
}


edge_t graph_get_edge(struct graph *g, int v1, int v2) {
	assert(v1 < g->n_vertices && v2 < g->n_vertices);
	return g->edges[v1 * g->a_vertices + v2];
}

void graph_set_edge(struct graph *g, int v1, int v2, edge_t e) {
	assert(v1 < g->n_vertices && v2 < g->n_vertices);
	g->edges[v1 * g->a_vertices + v2] = e;
}


iter_t graph_first_edge(struct graph *g, int v1) {
	return graph_next_edge(g, v1, -1);
}

int graph_iter_to_vertex(struct graph *g, int v1, iter_t i) {
	return i;
}

int graph_next_edge(struct graph *g, int v1, iter_t i) {
	int v2 = i;
	assert(v1 < g->n_vertices && v2 < g->n_vertices);
	/* XXX: Will the g->edges[] value carry over with optimizations? */
	for (int i = v2 + 1; i < g->n_vertices; i++)
		if (g->edges[v1 * g->a_vertices + i])
			return i;
	return -1;
}
