#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include <unistd.h>
#include "graph.h"

struct graph *graph_random(int vertices, int edges, int v_domain, int e_domain) {
	struct graph *g = graph_init((vertices + graph_alloc_step) & ~(graph_alloc_step - 1));
	for (int i = 0; i < vertices; i++) {
		int v = graph_add_vertex(g, 1 + (v_domain ? rand() % v_domain : 0));
		assert(v == i);
	}
	assert(edges < vertices * vertices);
	for (int i = 0; i < edges; i++) {
		int v1, v2;
		do {
			v1 = rand() % vertices;
			v2 = rand() % vertices;
		} while (unlikely(graph_get_edge(g, v1, v2)));
		graph_set_edge(g, v1, v2, 1 + (e_domain ? rand() % e_domain : 0));
	}
	return g;
}

#if 0
#define printf(x, ...)
#define puts(x)
#endif

void graph_print(struct graph *g) {
	printf("-- graph %p, %d vertices, %d allocated\n", g, g->n_vertices, g->a_vertices);
	for (int v = 0; v < g->n_vertices; v++) {
		printf("[%d] %d: ", v, graph_get_vertex(g, v));
		for (iter_t i = graph_first_edge(g, v); i >= 0; i = graph_next_edge(g, v, i)) {
			int v2 = graph_iter_to_vertex(g, v, i);
			printf("%d ", v2);
		}
		puts("");
	}
}

#undef printf
#undef puts


clock_t start_time;

void timestats(void) {
	struct tms t;
	clock_t now = times(&t);
	int u = sysconf(_SC_CLK_TCK);
	printf("user %fs, system %fs, total %fs\n",
			(float)t.tms_utime / u,
			(float)t.tms_stime / u,
			(float)(now - start_time) / u);
}
